@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.cc - API for DCRaw
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -35,7 +35,9 @@
 #define FORC4 FORC(4)
 #define FORCC FORC(colors)
 extern "C" {
-    int fcol_INDI(const unsigned filters, const int row, const int col);
+    int fcol_INDI(const unsigned filters, const int row, const int col,
+                  const int top_margin, const int left_margin,
+                  /*const*/ char xtrans[6][6]);
     void wavelet_denoise_INDI(gushort(*image)[4], const int black,
                               const int iheight, const int iwidth, const int height, const int width,
                               const int colors, const int shrink, const float pre_mul[4],
@@ -45,17 +47,22 @@ extern "C" {
                            float pre_mul[4], const unsigned filters, /*const*/ gushort white[8][8],
                            const char *ifname_display, void *dcraw);
     void lin_interpolate_INDI(gushort(*image)[4], const unsigned filters,
-                              const int width, const int height, const int colors, void *dcraw);
+                              const int width, const int height,
+                              const int colors, void *dcraw, dcraw_data *h);
     void vng_interpolate_INDI(gushort(*image)[4], const unsigned filters,
                               const int width, const int height, const int colors, const int rgb_max,
-                              void *dcraw);
+                              void *dcraw, dcraw_data *h);
+    void xtrans_interpolate_INDI(ushort(*image)[4], const unsigned filters,
+                                 const int width, const int height,
+                                 const int colors, const float rgb_cam[3][4],
+                                 void *dcraw, dcraw_data *hh, const int passes);
     void ahd_interpolate_INDI(gushort(*image)[4], const unsigned filters,
                               const int width, const int height, const int colors, float rgb_cam[3][4],
-                              void *dcraw);
+                              void *dcraw, dcraw_data *h);
     void color_smooth(gushort(*image)[4], const int width, const int height,
                       const int passes);
     void ppg_interpolate_INDI(gushort(*image)[4], const unsigned filters,
-                              const int width, const int height, const int colors, void *dcraw);
+                              const int width, const int height, const int colors, void *dcraw, dcraw_data *h);
     void flip_image_INDI(gushort(*image)[4], int *height_p, int *width_p,
                          const int flip);
     void fuji_rotate_INDI(gushort(**image_p)[4], int *height_p, int *width_p,
@@ -91,9 +98,6 @@ extern "C" {
             return DCRAW_OPEN_ERROR;
         }
         d->identify();
-#ifndef UFRAW_X_TRANS
-        if (d->filters == 2) d->is_raw = 0;
-#endif
         /* We first check if dcraw recognizes the file, this is equivalent
          * to 'dcraw -i' succeeding */
         if (!d->make[0]) {
@@ -131,6 +135,8 @@ extern "C" {
         h->colors = d->colors;
         h->filters = d->filters;
         h->raw_color = d->raw_color;
+        h->top_margin = d->top_margin;
+        h->left_margin = d->left_margin;
         memcpy(h->cam_mul, d->cam_mul, sizeof d->cam_mul);
         // maximum and black might change during load_raw. We need them for the
         // camera-wb. If they'll change we will recalculate the camera-wb.
@@ -139,8 +145,14 @@ extern "C" {
         FORC3 if ((unsigned)i > d->cblack[c]) i = d->cblack[c];
         FORC4 d->cblack[c] -= i;
         d->black += i;
+        i = d->cblack[6];
+        FORC(d->cblack[4] * d->cblack[5])
+        if (i > d->cblack[6 + c]) i = d->cblack[6 + c];
+        FORC(d->cblack[4] * d->cblack[5])
+        d->cblack[6 + c] -= i;
+        d->black += i;
         h->black = d->black;
-        h->shrink = d->shrink = (h->filters != 0);
+        h->shrink = d->shrink = (h->filters == 1 || h->filters > 1000);
         h->pixel_aspect = d->pixel_aspect;
         /* copied from dcraw's main() */
         switch ((d->flip + 3600) % 360) {
@@ -168,6 +180,7 @@ extern "C" {
         h->raw.image = NULL;
         h->thumbType = unknown_thumb_type;
         h->message = d->messageBuffer;
+        memcpy(h->xtrans, d->xtrans, sizeof d->xtrans);
         return d->lastStatus;
     }
 
@@ -215,15 +228,13 @@ extern "C" {
         }
         h->raw.height = d->iheight = (h->height + h->shrink) >> h->shrink;
         h->raw.width = d->iwidth = (h->width + h->shrink) >> h->shrink;
-        /* copied from the end of dcraw's identify() */
-        if (d->filters && d->colors == 3) {
-            d->filters |= ((d->filters >> 2 & 0x22222222) |
-                           (d->filters << 2 & 0x88888888)) & d->filters << 1;
-        }
         h->raw.colors = d->colors;
         h->fourColorFilters = d->filters;
         if (d->filters || d->colors == 1) {
-            d->raw_image = (ushort *) g_malloc((d->raw_height + 7) * d->raw_width * 2);
+            if (d->colors == 1 || d->filters == 1 || d->filters > 1000)
+                d->raw_image = (ushort *) g_malloc((d->raw_height + 7) * d->raw_width * 2);
+            else
+                d->raw_image = (ushort *) g_malloc(sizeof(dcraw_image_type) * (d->raw_height + 7) * d->raw_width);
         } else {
             h->raw.image = d->image = g_new0(dcraw_image_type, d->iheight * d->iwidth
                                              + d->meta_length);
@@ -243,6 +254,9 @@ extern "C" {
             d->meta_data = (char *)(d->image + d->iheight * d->iwidth);
             d->crop_masked_pixels();
             g_free(d->raw_image);
+
+            if (d->filters > 1 && d->filters <= 1000)
+                lin_interpolate_INDI(d->image, d->filters, d->width, d->height, d->colors, d, h);
         }
         if (!--d->data_error) d->lastStatus = DCRAW_ERROR;
         if (d->zero_is_bad) d->remove_zeroes();
@@ -264,6 +278,12 @@ extern "C" {
         FORC3 if ((unsigned)i > d->cblack[c]) i = d->cblack[c];
         FORC4 d->cblack[c] -= i;
         d->black += i;
+        i = d->cblack[6];
+        FORC(d->cblack[4] * d->cblack[5])
+        if (i > d->cblack[6 + c]) i = d->cblack[6 + c];
+        FORC(d->cblack[4] * d->cblack[5])
+        d->cblack[6 + c] -= i;
+        d->black += i;
         h->black = d->black;
         d->dcraw_message(DCRAW_VERBOSE, _("Black: %d, Maximum: %d\n"),
                          d->black, d->maximum);
@@ -361,13 +381,14 @@ extern "C" {
      * always try to index the column and not the row in order to reduce the
      * data cache footprint.
      */
-    static unsigned fcol_sequence(int filters, int row)
+    static unsigned fcol_sequence(int filters, int row, int top_margin,
+                                  int left_margin, char xtrans[6][6])
     {
         unsigned sequence = 0;
         int c;
 
         for (c = 15; c >= 0; --c)
-            sequence = (sequence << 2) | fcol_INDI(filters, row, c);
+            sequence = (sequence << 2) | fcol_INDI(filters, row, c, top_margin, left_margin, xtrans);
         return sequence;
     }
 
@@ -458,7 +479,7 @@ extern "C" {
         /* hh->raw.image is shrunk in half if there are filters.
          * If scale is odd we need to "unshrink" it using the info in
          * hh->fourColorFilters before scaling it. */
-        if (hh->filters != 0 && scale % 2 == 1) {
+        if ((hh->filters == 1 || hh->filters > 1000) && scale % 2 == 1) {
             fujiWidth = hh->fuji_width / scale;
             f->image = (dcraw_image_type *)
                        g_realloc(f->image, h * w * sizeof(dcraw_image_type));
@@ -470,7 +491,7 @@ extern "C" {
             for (r = 0; r < h; ++r) {
                 fseq = (unsigned*) g_malloc(scale * sizeof(unsigned));
                 for (ri = 0; ri < scale; ++ri)
-                    fseq[ri] = fcol_sequence(f4, r + ri);
+                    fseq[ri] = fcol_sequence(f4, r + ri, hh->top_margin, hh->left_margin, hh->xtrans);
                 for (c = 0; c < w; ++c) {
                     pixp = f->image[r * w + c];
                     shrink_pixel(pixp, r, c, hh, fseq, scale);
@@ -480,7 +501,7 @@ extern "C" {
                 g_free(fseq);
             }
         } else {
-            if (hh->filters != 0) scale /= 2;
+            if (hh->filters == 1 || hh->filters > 1000) scale /= 2;
             fujiWidth = ((hh->fuji_width + hh->shrink) >> hh->shrink) / scale;
             f->image = (dcraw_image_type *)g_realloc(
                            f->image, h * w * sizeof(dcraw_image_type));
@@ -723,33 +744,43 @@ extern "C" {
 
         /* It might be better to report an error here: */
         /* (dcraw also forbids AHD for Fuji rotated images) */
-        if (interpolation == dcraw_ahd_interpolation && (h->colors > 3 || h->filters < 1000))
+        if (h->filters == 9 && interpolation != dcraw_bilinear_interpolation)
+            interpolation = dcraw_xtrans_interpolation;
+        if (interpolation == dcraw_ahd_interpolation && h->colors > 3)
             interpolation = dcraw_vng_interpolation;
-        if (interpolation == dcraw_ppg_interpolation && (h->colors > 3 || h->filters < 1000))
+        if (interpolation == dcraw_ppg_interpolation && h->colors > 3)
             interpolation = dcraw_vng_interpolation;
         f4 = h->fourColorFilters;
-        for (r = 0; r < h->height; r++)
-            for (c = 0; c < h->width; c++) {
-                int cc = fcol_INDI(f4, r, c);
-                f->image[r * f->width + c][fcol_INDI(ff, r, c)] =
-                    h->raw.image[r / 2 * h->raw.width + c / 2][cc];
-            }
+        if (h->filters == 1 || h->filters > 1000) {
+            for (r = 0; r < h->height; r++)
+                for (c = 0; c < h->width; c++) {
+                    int cc = fcol_INDI(f4, r, c, h->top_margin, h->left_margin, h->xtrans);
+                    f->image[r * f->width + c][fcol_INDI(ff, r, c, h->top_margin, h->left_margin, h->xtrans)] =
+                        h->raw.image[r / 2 * h->raw.width + c / 2][cc];
+                }
+        } else
+            memcpy(f->image, h->raw.image, h->height * h->width * sizeof(dcraw_image_type));
         int smoothPasses = 1;
-        if (interpolation == dcraw_bilinear_interpolation)
-            lin_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
+        if (interpolation == dcraw_bilinear_interpolation && (h->filters == 1 || h->filters > 1000))
+            lin_interpolate_INDI(f->image, ff, f->width, f->height, cl, d, h);
 #ifdef ENABLE_INTERP_NONE
         else if (interpolation == dcraw_none_interpolation)
             smoothing = 0;
 #endif
-        else if (interpolation == dcraw_vng_interpolation)
-            vng_interpolate_INDI(f->image, ff, f->width, f->height, cl, 0xFFFF, d);
-        else if (interpolation == dcraw_ahd_interpolation) {
-            ahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
-                                 h->rgb_cam, d);
-            smoothPasses = 3;
-        } else if (interpolation == dcraw_ppg_interpolation)
-            ppg_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
+        else if (interpolation == dcraw_vng_interpolation || h->colors > 3)
+            vng_interpolate_INDI(f->image, ff, f->width, f->height, cl, 0xFFFF, d, h);
+        else if (interpolation == dcraw_ppg_interpolation && h->filters > 1000)
+            ppg_interpolate_INDI(f->image, ff, f->width, f->height, cl, d, h);
 
+        else if (interpolation == dcraw_xtrans_interpolation) {
+            xtrans_interpolate_INDI(f->image, h->filters, f->width, f->height,
+                                    h->colors, h->rgb_cam, d, h, 3);
+            smoothPasses = 3;
+        } else if (interpolation == dcraw_ahd_interpolation) {
+            ahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
+                                 h->rgb_cam, d, h);
+            smoothPasses = 3;
+        }
         if (smoothing)
             color_smooth(f->image, f->width, f->height, smoothPasses);
 
