@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_indi.c - DCRaw functions made independent
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -36,9 +36,9 @@
 #define uf_omp_get_num_threads() 1
 #endif
 
-#define ushort UshORt
-typedef unsigned short ushort;
-typedef gint64 INT64;
+#if !defined(ushort)
+#define ushort unsigned short
+#endif
 
 extern const double xyz_rgb[3][3];
 extern const float d65_white[3];
@@ -69,7 +69,9 @@ extern const float d65_white[3];
 #define BAYER(row,col) \
     image[((row) >> shrink)*iwidth + ((col) >> shrink)][FC(row,col)]
 
-int CLASS fcol_INDI(const unsigned filters, const int row, const int col)
+int CLASS fcol_INDI(const unsigned filters, const int row, const int col,
+                    const int top_margin, const int left_margin,
+                    /*const*/ char xtrans[6][6])
 {
     static const char filter[16][16] = {
         { 2, 1, 1, 3, 2, 3, 2, 0, 3, 2, 3, 0, 1, 2, 1, 0 },
@@ -89,24 +91,9 @@ int CLASS fcol_INDI(const unsigned filters, const int row, const int col)
         { 2, 1, 3, 2, 3, 1, 2, 1, 0, 3, 0, 2, 0, 2, 0, 2 },
         { 0, 3, 1, 0, 0, 2, 0, 3, 2, 1, 3, 1, 1, 3, 1, 3 }
     };
-#ifdef UFRAW_X_TRANS
-    static const char filter2[6][6] = {
-        { 1, 1, 0, 1, 1, 2 },
-        { 1, 1, 2, 1, 1, 0 },
-        { 2, 0, 1, 0, 2, 1 },
-        { 1, 1, 2, 1, 1, 0 },
-        { 1, 1, 0, 1, 1, 2 },
-        { 0, 2, 1, 2, 0, 1 }
-    };
-#endif
 
-    /* Assume that we are handling the Leaf CatchLight with
-     * top_margin = 8; left_margin = 18; */
-//  if (filters == 1) return filter[(row+top_margin) & 15][(col+left_margin) & 15];
-    if (filters == 1) return filter[(row + 8) & 15][(col + 18) & 15];
-#ifdef UFRAW_X_TRANS
-    if (filters == 2) return filter2[(row + 6) % 6][(col + 6) % 6];
-#endif
+    if (filters == 1) return filter[(row + top_margin) & 15][(col + left_margin) & 15];
+    if (filters == 9) return xtrans[(row + 6) % 6][(col + 6) % 6];
     return FC(row, col);
 }
 
@@ -250,6 +237,7 @@ void CLASS scale_colors_INDI(const int maximum, const int black,
         dcraw_message(dcraw, DCRAW_NO_CAMERA_WB,
                       _("%s: Cannot use camera white balance.\n"), ifname_display);
     }
+    if (pre_mul[1] == 0) pre_mul[1] = 1;
     if (pre_mul[3] == 0) pre_mul[3] = colors < 4 ? pre_mul[1] : 1;
     for (dmin = DBL_MAX, dmax = c = 0; c < 4; c++) {
         if (dmin > pre_mul[c])
@@ -268,7 +256,7 @@ void CLASS scale_colors_INDI(const int maximum, const int black,
 }
 
 void CLASS border_interpolate_INDI(const int height, const int width,
-                                   ushort(*image)[4], const unsigned filters, int colors, int border)
+                                   ushort(*image)[4], const unsigned filters, int colors, int border, dcraw_data *h)
 {
     int row, col, y, x, f, c, sum[8];
 
@@ -280,37 +268,35 @@ void CLASS border_interpolate_INDI(const int height, const int width,
             for (y = row - 1; y != row + 2; y++)
                 for (x = col - 1; x != col + 2; x++)
                     if (y >= 0 && y < height && x >= 0 && x < width) {
-                        f = fcol_INDI(filters, y, x);
+                        f = fcol_INDI(filters, y, x, h->top_margin, h->left_margin, h->xtrans);
                         sum[f] += image[y * width + x][f];
                         sum[f + 4]++;
                     }
-            f = fcol_INDI(filters, row, col);
+            f = fcol_INDI(filters, row, col, h->top_margin, h->left_margin, h->xtrans);
             FORCC if (c != f && sum[c + 4])
                 image[row * width + col][c] = sum[c] / sum[c + 4];
         }
 }
 
 void CLASS lin_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-                                const int width, const int height, const int colors, void *dcraw) /*UF*/
+                                const int width, const int height, const int colors, void *dcraw, dcraw_data *h) /*UF*/
 {
     int code[16][16][32], size = 16, *ip, sum[4];
     int f, c, i, x, y, row, col, shift, color;
     ushort *pix;
 
     dcraw_message(dcraw, DCRAW_VERBOSE, _("Bilinear interpolation...\n")); /*UF*/
-#ifdef UFRAW_X_TRANS
-    if (filters == 2) size = 6;
-#endif
-    border_interpolate_INDI(height, width, image, filters, colors, 1);
+    if (filters == 9) size = 6;
+    border_interpolate_INDI(height, width, image, filters, colors, 1, h);
     for (row = 0; row < size; row++) {
         for (col = 0; col < size; col++) {
             ip = code[row][col] + 1;
-            f = fcol_INDI(filters, row, col);
+            f = fcol_INDI(filters, row, col, h->top_margin, h->left_margin, h->xtrans);
             memset(sum, 0, sizeof sum);
             for (y = -1; y <= 1; y++)
                 for (x = -1; x <= 1; x++) {
                     shift = (y == 0) + (x == 0);
-                    color = fcol_INDI(filters, row + y, col + x);
+                    color = fcol_INDI(filters, row + y, col + x, h->top_margin, h->left_margin, h->xtrans);
                     if (color == f) continue;
                     *ip++ = (width * y + x) * 4 + color;
                     *ip++ = shift;
@@ -352,7 +338,7 @@ void CLASS lin_interpolate_INDI(ushort(*image)[4], const unsigned filters,
    Gradients are numbered clockwise from NW=0 to W=7.
  */
 void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-                                const int width, const int height, const int colors, void *dcraw) /*UF*/
+                                const int width, const int height, const int colors, void *dcraw, dcraw_data *h) /*UF*/
 {
     static const signed char *cp, terms[] = {
         -2, -2, +0, -1, 0, 0x01, -2, -2, +0, +0, 1, 0x01, -2, -1, -1, +0, 0, 0x01,
@@ -384,13 +370,11 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
     int g, diff, thold, num, c;
     ushort rowtmp[4][width * 4];
 
-    lin_interpolate_INDI(image, filters, width, height, colors, dcraw); /*UF*/
+    lin_interpolate_INDI(image, filters, width, height, colors, dcraw, h); /*UF*/
     dcraw_message(dcraw, DCRAW_VERBOSE, _("VNG interpolation...\n")); /*UF*/
 
     if (filters == 1) prow = pcol = 16;
-#ifdef UFRAW_X_TRANS
-    if (filters == 2) prow = pcol =  6;
-#endif
+    if (filters == 9) prow = pcol =  6;
     int *ipalloc = ip = (int *) calloc(prow * pcol, 1280);
     merror(ip, "vng_interpolate()");
     for (row = 0; row < prow; row++)		/* Precalculate for VNG */
@@ -403,9 +387,9 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 x2 = *cp++;
                 weight = *cp++;
                 grads = *cp++;
-                color = fcol_INDI(filters, row + y1, col + x1);
-                if (fcol_INDI(filters, row + y2, col + x2) != color) continue;
-                diag = (fcol_INDI(filters, row, col + 1) == color && fcol_INDI(filters, row + 1, col) == color) ? 2 : 1;
+                color = fcol_INDI(filters, row + y1, col + x1, h->top_margin, h->left_margin, h->xtrans);
+                if (fcol_INDI(filters, row + y2, col + x2, h->top_margin, h->left_margin, h->xtrans) != color) continue;
+                diag = (fcol_INDI(filters, row, col + 1, h->top_margin, h->left_margin, h->xtrans) == color && fcol_INDI(filters, row + 1, col, h->top_margin, h->left_margin, h->xtrans) == color) ? 2 : 1;
                 if (abs(y1 - y2) == diag && abs(x1 - x2) == diag) continue;
                 *ip++ = (y1 * width + x1) * 4 + color;
                 *ip++ = (y2 * width + x2) * 4 + color;
@@ -419,8 +403,8 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 y = *cp++;
                 x = *cp++;
                 *ip++ = (y * width + x) * 4;
-                color = fcol_INDI(filters, row, col);
-                if (fcol_INDI(filters, row + y, col + x) != color && fcol_INDI(filters, row + y * 2, col + x * 2) == color)
+                color = fcol_INDI(filters, row, col, h->top_margin, h->left_margin, h->xtrans);
+                if (fcol_INDI(filters, row + y, col + x, h->top_margin, h->left_margin, h->xtrans) != color && fcol_INDI(filters, row + y * 2, col + x * 2, h->top_margin, h->left_margin, h->xtrans) == color)
                     *ip++ = (y * width + x) * 8 + color;
                 else
                     *ip++ = 0;
@@ -430,7 +414,7 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
 #ifdef _OPENMP
     #pragma omp parallel				\
     default(none)					\
-    shared(image,code,prow,pcol)			\
+    shared(image,code,prow,pcol,h)			\
     private(row,col,g,brow,rowtmp,pix,ip,gval,diff,gmin,gmax,thold,sum,color,num,c,t)
 #endif
     {
@@ -466,7 +450,7 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 }
                 thold = gmin + (gmax >> 1);
                 memset(sum, 0, sizeof sum);
-                color = fcol_INDI(filters, row, col);
+                color = fcol_INDI(filters, row, col, h->top_margin, h->left_margin, h->xtrans);
                 for (num = g = 0; g < 8; g++, ip += 2) { /* Average the neighbors */
                     if (gval[g] <= thold) {
                         FORCC
@@ -501,20 +485,20 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
 */
 void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                                 const int width, const int height,
-                                const int colors, void *dcraw)
+                                const int colors, void *dcraw, dcraw_data *h)
 {
     int dir[5] = { 1, width, -1, -width, 1 };
-    int row, col, diff[2], guess[2], c, d, i;
+    int row, col, diff[2] = { 0, 0 }, guess[2], c, d, i;
     ushort(*pix)[4];
 
-    border_interpolate_INDI(height, width, image, filters, colors, 3);
+    border_interpolate_INDI(height, width, image, filters, colors, 3, h);
     dcraw_message(dcraw, DCRAW_VERBOSE, _("PPG interpolation...\n")); /*UF*/
 
 #ifdef _OPENMP
     #pragma omp parallel				\
     default(none)					\
-    shared(image,dir)					\
-    private(row,col,i,d,c,pix,diff,guess)
+    shared(image,dir,diff)				\
+    private(row,col,i,d,c,pix,guess)
 #endif
     {
         /*  Fill in the green layer with gradients and pattern recognition: */
@@ -572,23 +556,306 @@ void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
     }
 }
 
+void CLASS cielab_INDI(ushort rgb[3], short lab[3], const int colors,
+                       const float rgb_cam[3][4])
+{
+    int c, i, j, k;
+    float r, xyz[3];
+    static float cbrt[0x10000], xyz_cam[3][4];
+
+    if (!rgb) {
+        for (i = 0; i < 0x10000; i++) {
+            r = i / 65535.0;
+            cbrt[i] = r > 0.008856 ? pow(r, (float)(1 / 3.0)) : 7.787 * r + 16 / 116.0;
+        }
+        for (i = 0; i < 3; i++)
+            for (j = 0; j < colors; j++)
+                for (xyz_cam[i][j] = k = 0; k < 3; k++)
+                    xyz_cam[i][j] += xyz_rgb[i][k] * rgb_cam[k][j] / d65_white[i];
+        return;
+    }
+    xyz[0] = xyz[1] = xyz[2] = 0.5;
+    FORCC {
+        xyz[0] += xyz_cam[0][c] * rgb[c];
+        xyz[1] += xyz_cam[1][c] * rgb[c];
+        xyz[2] += xyz_cam[2][c] * rgb[c];
+    }
+    xyz[0] = cbrt[CLIP((int) xyz[0])];
+    xyz[1] = cbrt[CLIP((int) xyz[1])];
+    xyz[2] = cbrt[CLIP((int) xyz[2])];
+    lab[0] = 64 * (116 * xyz[1] - 16);
+    lab[1] = 64 * 500 * (xyz[0] - xyz[1]);
+    lab[2] = 64 * 200 * (xyz[1] - xyz[2]);
+}
+
+#define TS 512		/* Tile Size */
+/*
+   Frank Markesteijn's algorithm for Fuji X-Trans sensors
+ */
+void CLASS xtrans_interpolate_INDI(ushort(*image)[4], const unsigned filters,
+                                   const int width, const int height,
+                                   const int colors, const float rgb_cam[3][4],
+                                   void *dcraw, dcraw_data *hh, const int passes)
+{
+    int c, d, f, g, h, i, v, ng, row, col, top, left, mrow, mcol;
+    int val, ndir, pass, hm[8], avg[4], color[3][8];
+    static const short orth[12] = { 1, 0, 0, 1, -1, 0, 0, -1, 1, 0, 0, 1 },
+    patt[2][16] = { { 0, 1, 0, -1, 2, 0, -1, 0, 1, 1, 1, -1, 0, 0, 0, 0 },
+        { 0, 1, 0, -2, 1, 0, -2, 0, 1, 1, -2, -2, 1, -1, -1, 1 }
+    },
+    dir[4] = { 1, TS, TS + 1, TS - 1 };
+    short allhex[3][3][2][8], *hex;
+    ushort min, max, sgrow = 0, sgcol = 0;
+    ushort(*rgb)[TS][TS][3], (*rix)[3], (*pix)[4];
+    short(*lab)    [TS][3], (*lix)[3];
+    float(*drv)[TS][TS], diff[6], tr;
+    char(*homo)[TS][TS], *buffer;
+
+    dcraw_message(dcraw, DCRAW_VERBOSE, _("%d-pass X-Trans interpolation...\n"), passes); /*NKBJ*/
+
+    cielab_INDI(0, 0, colors, rgb_cam);
+    border_interpolate_INDI(height, width, image, filters, colors, 6, hh);
+    ndir = 4 << (passes > 1);
+
+    /* Map a green hexagon around each non-green pixel and vice versa:      */
+    for (row = 0; row < 3; row++)
+        for (col = 0; col < 3; col++)
+            for (ng = d = 0; d < 10; d += 2) {
+                g = fcol_INDI(filters, row, col, hh->top_margin, hh->left_margin, hh->xtrans) == 1;
+                if (fcol_INDI(filters, row + orth[d], col + orth[d + 2], hh->top_margin, hh->left_margin, hh->xtrans) == 1) ng = 0;
+                else ng++;
+                if (ng == 4) {
+                    sgrow = row;
+                    sgcol = col;
+                }
+                if (ng == g + 1) FORC(8) {
+                    v = orth[d  ] * patt[g][c * 2] + orth[d + 1] * patt[g][c * 2 + 1];
+                    h = orth[d + 2] * patt[g][c * 2] + orth[d + 3] * patt[g][c * 2 + 1];
+                    allhex[row][col][0][c ^ (g * 2 & d)] = h + v * width;
+                    allhex[row][col][1][c ^ (g * 2 & d)] = h + v * TS;
+                }
+            }
+
+    /* Set green1 and green3 to the minimum and maximum allowed values:     */
+    for (row = 2; row < height - 2; row++)
+        for (min = ~(max = 0), col = 2; col < width - 2; col++) {
+            if (fcol_INDI(filters, row, col, hh->top_margin, hh->left_margin, hh->xtrans) == 1 && (min = ~(max = 0))) continue;
+            pix = image + row * width + col;
+            hex = allhex[row % 3][col % 3][0];
+            if (!max) FORC(6) {
+                val = pix[hex[c]][1];
+                if (min > val) min = val;
+                if (max < val) max = val;
+            }
+            pix[0][1] = min;
+            pix[0][3] = max;
+            switch ((row - sgrow) % 3) {
+                case 1:
+                    if (row < height - 3) {
+                        row++;
+                        col--;
+                    }
+                    break;
+                case 2:
+                    if ((min = ~(max = 0)) && (col += 2) < width - 3 && row > 2) row--;
+            }
+        }
+
+
+#ifdef _OPENMP
+    #pragma omp parallel				\
+    default(shared)					\
+    private(top, left, row, col, pix, mrow, mcol, hex, color, c, pass, rix, val, d, f, g, h, i, diff, lix, tr, avg, v, buffer, rgb, lab, drv, homo, hm, max)
+#endif
+    {
+        buffer = (char *) malloc(TS * TS * (ndir * 11 + 6));
+        merror(buffer, "xtrans_interpolate()");
+        rgb  = (ushort(*)[TS][TS][3]) buffer;
+        lab  = (short(*)    [TS][3])(buffer + TS * TS * (ndir * 6));
+        drv  = (float(*)[TS][TS])(buffer + TS * TS * (ndir * 6 + 6));
+        homo = (char(*)[TS][TS])(buffer + TS * TS * (ndir * 10 + 6));
+
+        progress(PROGRESS_INTERPOLATE, -height);
+
+#ifdef _OPENMP
+        #pragma omp for
+#endif
+
+        for (top = 3; top < height - 19; top += TS - 16) {
+            progress(PROGRESS_INTERPOLATE, TS - 16);
+            for (left = 3; left < width - 19; left += TS - 16) {
+                mrow = MIN(top + TS, height - 3);
+                mcol = MIN(left + TS, width - 3);
+                for (row = top; row < mrow; row++)
+                    for (col = left; col < mcol; col++)
+                        memcpy(rgb[0][row - top][col - left], image[row * width + col], 6);
+                FORC3 memcpy(rgb[c + 1], rgb[0], sizeof * rgb);
+
+                /* Interpolate green horizontally, vertically, and along both diagonals: */
+                for (row = top; row < mrow; row++)
+                    for (col = left; col < mcol; col++) {
+                        if ((f = fcol_INDI(filters, row, col, hh->top_margin, hh->left_margin, hh->xtrans)) == 1) continue;
+                        pix = image + row * width + col;
+                        hex = allhex[row % 3][col % 3][0];
+                        color[1][0] = 174 * (pix[  hex[1]][1] + pix[  hex[0]][1]) -
+                                      46 * (pix[2 * hex[1]][1] + pix[2 * hex[0]][1]);
+                        color[1][1] = 223 *  pix[  hex[3]][1] + pix[  hex[2]][1] * 33 +
+                                      92 * (pix[      0 ][f] - pix[ -hex[2]][f]);
+                        FORC(2) color[1][2 + c] =
+                            164 * pix[hex[4 + c]][1] + 92 * pix[-2 * hex[4 + c]][1] + 33 *
+                            (2 * pix[0][f] - pix[3 * hex[4 + c]][f] - pix[-3 * hex[4 + c]][f]);
+                        FORC4 rgb[c ^ !((row - sgrow) % 3)][row - top][col - left][1] =
+                            LIM(color[1][c] >> 8, pix[0][1], pix[0][3]);
+                    }
+
+                for (pass = 0; pass < passes; pass++) {
+                    if (pass == 1)
+                        memcpy(rgb += 4, buffer, 4 * sizeof * rgb);
+
+                    /* Recalculate green from interpolated values of closer pixels: */
+                    if (pass) {
+                        for (row = top + 2; row < mrow - 2; row++)
+                            for (col = left + 2; col < mcol - 2; col++) {
+                                if ((f = fcol_INDI(filters, row, col, hh->top_margin, hh->left_margin, hh->xtrans)) == 1) continue;
+                                pix = image + row * width + col;
+                                hex = allhex[row % 3][col % 3][1];
+                                for (d = 3; d < 6; d++) {
+                                    rix = &rgb[(d - 2) ^ !((row - sgrow) % 3)][row - top][col - left];
+                                    val = rix[-2 * hex[d]][1] + 2 * rix[hex[d]][1]
+                                          - rix[-2 * hex[d]][f] - 2 * rix[hex[d]][f] + 3 * rix[0][f];
+                                    rix[0][1] = LIM(val / 3, pix[0][1], pix[0][3]);
+                                }
+                            }
+                    }
+
+                    /* Interpolate red and blue values for solitary green pixels:   */
+                    for (row = (top - sgrow + 4) / 3 * 3 + sgrow; row < mrow - 2; row += 3)
+                        for (col = (left - sgcol + 4) / 3 * 3 + sgcol; col < mcol - 2; col += 3) {
+                            rix = &rgb[0][row - top][col - left];
+                            h = fcol_INDI(filters, row, col + 1, hh->top_margin, hh->left_margin, hh->xtrans);
+                            memset(diff, 0, sizeof diff);
+                            for (i = 1, d = 0; d < 6; d++, i ^= TS ^ 1, h ^= 2) {
+                                for (c = 0; c < 2; c++, h ^= 2) {
+                                    g = 2 * rix[0][1] - rix[i << c][1] - rix[-i << c][1];
+                                    color[h][d] = g + rix[i << c][h] + rix[-i << c][h];
+                                    if (d > 1)
+                                        diff[d] += SQR(rix[i << c][1] - rix[-i << c][1]
+                                                       - rix[i << c][h] + rix[-i << c][h]) + SQR(g);
+                                }
+                                if (d > 1 && (d & 1))
+                                    if (diff[d - 1] < diff[d])
+                                        FORC(2) color[c * 2][d] = color[c * 2][d - 1];
+                                if (d < 2 || (d & 1)) {
+                                    FORC(2) rix[0][c * 2] = CLIP(color[c * 2][d] / 2);
+                                    rix += TS * TS;
+                                }
+                            }
+                        }
+
+                    /* Interpolate red for blue pixels and vice versa:              */
+                    for (row = top + 1; row < mrow - 1; row++)
+                        for (col = left + 1; col < mcol - 1; col++) {
+                            if ((f = 2 - fcol_INDI(filters, row, col, hh->top_margin, hh->left_margin, hh->xtrans)) == 1) continue;
+                            rix = &rgb[0][row - top][col - left];
+                            i = (row - sgrow) % 3 ? TS : 1;
+                            for (d = 0; d < 4; d++, rix += TS * TS)
+                                rix[0][f] = CLIP((rix[i][f] + rix[-i][f] +
+                                                  2 * rix[0][1] - rix[i][1] - rix[-i][1]) / 2);
+                        }
+
+                    /* Fill in red and blue for 2x2 blocks of green:                */
+                    for (row = top + 2; row < mrow - 2; row++) if ((row - sgrow) % 3)
+                            for (col = left + 2; col < mcol - 2; col++) if ((col - sgcol) % 3) {
+                                    rix = &rgb[0][row - top][col - left];
+                                    hex = allhex[row % 3][col % 3][1];
+                                    for (d = 0; d < ndir; d += 2, rix += TS * TS)
+                                        if (hex[d] + hex[d + 1]) {
+                                            g = 3 * rix[0][1] - 2 * rix[hex[d]][1] - rix[hex[d + 1]][1];
+                                            for (c = 0; c < 4; c += 2) rix[0][c] =
+                                                    CLIP((g + 2 * rix[hex[d]][c] + rix[hex[d + 1]][c]) / 3);
+                                        } else {
+                                            g = 2 * rix[0][1] - rix[hex[d]][1] - rix[hex[d + 1]][1];
+                                            for (c = 0; c < 4; c += 2) rix[0][c] =
+                                                    CLIP((g + rix[hex[d]][c] + rix[hex[d + 1]][c]) / 2);
+                                        }
+                                }
+                }
+                rgb = (ushort(*)[TS][TS][3]) buffer;
+                mrow -= top;
+                mcol -= left;
+
+                /* Convert to CIELab and differentiate in all directions:       */
+                for (d = 0; d < ndir; d++) {
+                    for (row = 2; row < mrow - 2; row++)
+                        for (col = 2; col < mcol - 2; col++)
+                            cielab_INDI(rgb[d][row][col], lab[row][col], colors, rgb_cam);
+                    for (f = dir[d & 3], row = 3; row < mrow - 3; row++)
+                        for (col = 3; col < mcol - 3; col++) {
+                            lix = &lab[row][col];
+                            g = 2 * lix[0][0] - lix[f][0] - lix[-f][0];
+                            drv[d][row][col] = SQR(g)
+                                               + SQR((2 * lix[0][1] - lix[f][1] - lix[-f][1] + g * 500 / 232))
+                                               + SQR((2 * lix[0][2] - lix[f][2] - lix[-f][2] - g * 500 / 580));
+                        }
+                }
+
+                /* Build homogeneity maps from the derivatives:                 */
+                memset(homo, 0, ndir * TS * TS);
+                for (row = 4; row < mrow - 4; row++)
+                    for (col = 4; col < mcol - 4; col++) {
+                        for (tr = FLT_MAX, d = 0; d < ndir; d++)
+                            if (tr > drv[d][row][col])
+                                tr = drv[d][row][col];
+                        tr *= 8;
+                        for (d = 0; d < ndir; d++)
+                            for (v = -1; v <= 1; v++)
+                                for (h = -1; h <= 1; h++)
+                                    if (drv[d][row + v][col + h] <= tr)
+                                        homo[d][row][col]++;
+                    }
+
+                /* Average the most homogenous pixels for the final result:     */
+                if (height - top < TS + 4) mrow = height - top + 2;
+                if (width - left < TS + 4) mcol = width - left + 2;
+                for (row = MIN(top, 8); row < mrow - 8; row++)
+                    for (col = MIN(left, 8); col < mcol - 8; col++) {
+                        for (d = 0; d < ndir; d++)
+                            for (hm[d] = 0, v = -2; v <= 2; v++)
+                                for (h = -2; h <= 2; h++)
+                                    hm[d] += homo[d][row + v][col + h];
+                        for (d = 0; d < ndir - 4; d++)
+                            if (hm[d] < hm[d + 4]) hm[d  ] = 0;
+                            else if (hm[d] > hm[d + 4]) hm[d + 4] = 0;
+                        for (max = hm[0], d = 1; d < ndir; d++)
+                            if (max < hm[d]) max = hm[d];
+                        max -= max >> 3;
+                        memset(avg, 0, sizeof avg);
+                        for (d = 0; d < ndir; d++)
+                            if (hm[d] >= max) {
+                                FORC3 avg[c] += rgb[d][row][col][c];
+                                avg[3]++;
+                            }
+                        FORC3 image[(row + top)*width + col + left][c] = avg[c] / avg[3];
+                    }
+            }
+        }
+        free(buffer);
+    } /* _OPENMP */
+}
+
 /*
    Adaptive Homogeneity-Directed interpolation is based on
    the work of Keigo Hirakawa, Thomas Parks, and Paul Lee.
  */
-#define TS 256 /* Tile Size */
-
 void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                                 const int width, const int height,
                                 const int colors, const float rgb_cam[3][4],
-                                void *dcraw)
+                                void *dcraw, dcraw_data *h)
 {
-    int i, j, k, top, left, row, col, tr, tc, c, d, val, hm[2];
-    ushort(*pix)[4], (*rix)[3];
+    int i, j, top, left, row, col, tr, tc, c, d, val, hm[2];
     static const int dir[4] = { -1, 1, -TS, TS };
     unsigned ldiff[2][4], abdiff[2][4], leps, abeps;
-    float r, cbrt[0x10000], xyz[3], xyz_cam[3][4];
-    ushort(*rgb)[TS][TS][3];
+    ushort(*rgb)[TS][TS][3], (*rix)[3], (*pix)[4];
     short(*lab)[TS][TS][3], (*lix)[3];
     char(*homo)[TS][TS], *buffer;
 
@@ -597,26 +864,12 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
 #ifdef _OPENMP
     #pragma omp parallel				\
     default(shared)					\
-    private(top, left, row, col, pix, rix, lix, c, xyz, val, d, tc, tr, i, j, k, ldiff, abdiff, leps, abeps, hm, buffer, rgb, lab, homo, r)
+    private(top, left, row, col, pix, rix, lix, c, val, d, tc, tr, i, j, ldiff, abdiff, leps, abeps, hm, buffer, rgb, lab, homo)
 #endif
     {
-#ifdef _OPENMP
-        #pragma omp for schedule(static) nowait
-#endif
-        for (i = 0; i < 0x10000; i++) {
-            r = i / 65535.0;
-            cbrt[i] = r > 0.008856 ? pow(r, 1 / 3.0) : 7.787 * r + 16 / 116.0;
-        }
-#ifdef _OPENMP
-        #pragma omp for
-#endif
-        for (i = 0; i < 3; i++)
-            for (j = 0; j < colors; j++)
-                for (xyz_cam[i][j] = k = 0; k < 3; k++)
-                    xyz_cam[i][j] += xyz_rgb[i][k] * rgb_cam[k][j] / d65_white[i];
-
-        border_interpolate_INDI(height, width, image, filters, colors, 5);
-        buffer = (char *) malloc(26 * TS * TS); /* 1664 kB */
+        cielab_INDI(0, 0, colors, rgb_cam);
+        border_interpolate_INDI(height, width, image, filters, colors, 5, h);
+        buffer = (char *) malloc(26 * TS * TS);
         merror(buffer, "ahd_interpolate()");
         rgb  = (ushort(*)[TS][TS][3]) buffer;
         lab  = (short(*)[TS][TS][3])(buffer + 12 * TS * TS);
@@ -665,18 +918,7 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                             rix[0][c] = CLIP(val);
                             c = FC(row, col);
                             rix[0][c] = pix[0][c];
-                            xyz[0] = xyz[1] = xyz[2] = 0.5;
-                            FORCC {
-                                xyz[0] += xyz_cam[0][c] * rix[0][c];
-                                xyz[1] += xyz_cam[1][c] * rix[0][c];
-                                xyz[2] += xyz_cam[2][c] * rix[0][c];
-                            }
-                            xyz[0] = cbrt[CLIP((int) xyz[0])];
-                            xyz[1] = cbrt[CLIP((int) xyz[1])];
-                            xyz[2] = cbrt[CLIP((int) xyz[2])];
-                            lix[0][0] = 64 * (116 * xyz[1] - 16);
-                            lix[0][1] = 64 * 500 * (xyz[0] - xyz[1]);
-                            lix[0][2] = 64 * 200 * (xyz[1] - xyz[2]);
+                            cielab_INDI(rix[0], lix[0], colors, rgb_cam);
                         }
                 /*  Build homogeneity maps from the CIELab images: */
                 memset(homo, 0, 2 * TS * TS);
@@ -761,7 +1003,7 @@ static inline int median9(int *p)
     PIX_SORT(p[4], p[2]) ;
     PIX_SORT(p[6], p[4]) ;
     PIX_SORT(p[4], p[2]) ;
-    return(p[4]) ;
+    return (p[4]) ;
 }
 
 #undef PIX_SWAP
@@ -882,14 +1124,14 @@ void CLASS flip_image_INDI(ushort(*image)[4], int *height_p, int *width_p,
 {
     unsigned *flag;
     int size, base, dest, next, row, col;
-    INT64 *img, hold;
+    gint64 *img, hold;
     int height = *height_p, width = *width_p;/* INDI - UF*/
 
 //  Message is suppressed because error handling is not enabled here.
 //  dcraw_message (dcraw, DCRAW_VERBOSE,_("Flipping image %c:%c:%c...\n"),
 //      flip & 1 ? 'H':'0', flip & 2 ? 'V':'0', flip & 4 ? 'T':'0'); /*UF*/
 
-    img = (INT64 *) image;
+    img = (gint64 *) image;
     size = height * width;
     flag = calloc((size + 31) >> 5, sizeof * flag);
     merror(flag, "flip_image()");
