@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-gimp.c - The GIMP plug-in.
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * based on the GIMP plug-in by Pawel T. Jochym jochym at ifj edu pl,
  *
@@ -17,41 +17,19 @@
 
 #include "ufraw.h"
 #include "uf_gtk.h"
-#ifdef UFRAW_CINEPAINT
-/* Bypass a bug in CinePaint header files */
-#define RNH_FLOAT
-#include <plugin_main.h>
-#define GIMP_CONST
-/* Fix some compatibility issues between CinePaint and GIMP */
-typedef GimpRunModeType GimpRunMode;
-#define PLUGIN_MODE 2
-#define DEPTH_TO_BASETYPE(depth) (depth == 3 ? RGB : U16_RGB)
-#define DEPTH_TO_IMAGETYPE(depth) (depth == 3 ? RGB_IMAGE : U16_RGB_IMAGE)
-#else /* GIMP */
 #if HAVE_GIMP_2_9
 #include <gegl.h>
-#define PLUGIN_MODE 2
-#else
-#define PLUGIN_MODE 1
 #endif
 #include <libgimpbase/gimpbase.h>
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
-#define GIMP_CONST const
-/* Missing and irrelevant definitions in GIMP */
-#define U16_RGB 0
-#define U16_RGB_IMAGE 0
-#define DEPTH_TO_BASETYPE(depth) GIMP_RGB
-#define DEPTH_TO_IMAGETYPE(depth) GIMP_RGB_IMAGE
-#define DEPTH_TO_PRECISION(depth) (depth == 3 ? GIMP_PRECISION_U8 : GIMP_PRECISION_U16)
-#endif
 #include <glib/gi18n.h>
 #include <string.h>
 
 void query();
-void run(GIMP_CONST gchar *name,
+void run(const gchar *name,
          gint nparams,
-         GIMP_CONST GimpParam *param,
+         const GimpParam *param,
          gint *nreturn_vals,
          GimpParam **return_vals);
 
@@ -68,38 +46,32 @@ MAIN()
 
 void query()
 {
-    static GIMP_CONST GimpParamDef load_args[] = {
+    static const GimpParamDef load_args[] = {
         { GIMP_PDB_INT32,  "run_mode", "Interactive, non-interactive" },
         { GIMP_PDB_STRING, "filename", "The name of the file to load" },
         { GIMP_PDB_STRING, "raw_filename", "The name of the file to load" },
     };
-    static GIMP_CONST GimpParamDef load_return_vals[] = {
+    static const GimpParamDef load_return_vals[] = {
         { GIMP_PDB_IMAGE, "image", "Output image" },
     };
-#ifndef UFRAW_CINEPAINT
-    static GIMP_CONST GimpParamDef thumb_args[] = {
+    static const GimpParamDef thumb_args[] = {
         { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
         { GIMP_PDB_INT32,  "thumb_size",   "Preferred thumbnail size" }
     };
-    static GIMP_CONST GimpParamDef thumb_return_vals[] = {
+    static const GimpParamDef thumb_return_vals[] = {
         { GIMP_PDB_IMAGE,  "image",        "Thumbnail image" },
         { GIMP_PDB_INT32,  "image_width",  "Width of full-sized image" },
         { GIMP_PDB_INT32,  "image_height", "Height of full-sized image" }
     };
-#endif
     gimp_install_procedure("file_ufraw_load",
                            "Loads digital camera raw files",
                            "Loads digital camera raw files.",
                            "Udi Fuchs",
                            "Copyright 2003 by Dave Coffin\n"
                            "Copyright 2004 by Pawel Jochym\n"
-                           "Copyright 2004-2013 by Udi Fuchs",
+                           "Copyright 2004-2014 by Udi Fuchs",
                            "ufraw-" VERSION,
-#ifndef UFRAW_CINEPAINT
                            "raw image",
-#else
-                           "<Load>/UFRaw",
-#endif
                            NULL,
                            GIMP_PLUGIN,
                            G_N_ELEMENTS(load_args),
@@ -107,14 +79,18 @@ void query()
                            load_args,
                            load_return_vals);
 
-    gimp_register_load_handler("file_ufraw_load", (char *)raw_ext, "");
+    gimp_register_magic_load_handler("file_ufraw_load",
+                                     (char *)raw_ext,
+                                     "",
+                                     "0,string,II*\\0,"
+                                     "0,string,MM\\0*,"
+                                     "0,string,<?xml");
 
-#ifndef UFRAW_CINEPAINT
     gimp_install_procedure("file_ufraw_load_thumb",
                            "Loads thumbnails from digital camera raw files.",
                            "Loads thumbnails from digital camera raw files.",
                            "Udi Fuchs",
-                           "Copyright 2004-2013 by Udi Fuchs",
+                           "Copyright 2004-2014 by Udi Fuchs",
                            "ufraw-" VERSION,
                            NULL,
                            NULL,
@@ -125,25 +101,14 @@ void query()
 
     gimp_register_thumbnail_loader("file_ufraw_load",
                                    "file_ufraw_load_thumb");
-#endif
 }
 
 char *ufraw_binary;
-
-#ifdef UFRAW_CINEPAINT
-/* There is no way to get the full executable path in Cinepaint.
- * It is only needed for windows, so no big deal. */
-char *gimp_get_progname()
-{
-    return "ufraw-cinepaint";
-}
-#endif
-
 gboolean sendToGimpMode;
 
-void run(GIMP_CONST gchar *name,
+void run(const gchar *name,
          gint nparams,
-         GIMP_CONST GimpParam *param,
+         const GimpParam *param,
          gint *nreturn_vals,
          GimpParam **return_vals)
 {
@@ -160,8 +125,10 @@ void run(GIMP_CONST gchar *name,
 #if !GLIB_CHECK_VERSION(2,31,0)
     g_thread_init(NULL);
 #endif
+#ifndef _WIN32
     gdk_threads_init();
     gdk_threads_enter();
+#endif
     ufraw_binary = g_path_get_basename(gimp_get_progname());
     uf_init_locale(gimp_get_progname());
 #if HAVE_GIMP_2_9
@@ -182,7 +149,9 @@ void run(GIMP_CONST gchar *name,
     } else {
         values[0].type = GIMP_PDB_STATUS;
         values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
+#ifndef _WIN32
         gdk_threads_leave();
+#endif
         return;
     }
     gboolean loadThumbnail = size > 0;
@@ -203,7 +172,9 @@ void run(GIMP_CONST gchar *name,
             else
                 *return_vals = gimp_run_procedure2("file_jpeg_load",
                                                    nreturn_vals, nparams, param);
+#ifndef _WIN32
             gdk_threads_leave();
+#endif
             return;
         } else if (!strcasecmp(filename + strlen(filename) - 4, ".tif") ||
                    !strcasecmp(filename + strlen(filename) - 5, ".tiff")) {
@@ -225,13 +196,17 @@ void run(GIMP_CONST gchar *name,
                     	nreturn_vals, 3, tiffParam);
                 */
             }
+#ifndef _WIN32
             gdk_threads_leave();
+#endif
             return;
         } else {
             /* Don't issue a message on thumbnail failure, since ufraw-gimp
              * will be called again with "file_ufraw_load" */
             if (loadThumbnail) {
+#ifndef _WIN32
                 gdk_threads_leave();
+#endif
                 return;
             }
             ufraw_icons_init();
@@ -245,7 +220,9 @@ void run(GIMP_CONST gchar *name,
             values[0].data.d_status = GIMP_PDB_CANCEL;
 
             gtk_widget_destroy(dummyWindow);
+#ifndef _WIN32
             gdk_threads_leave();
+#endif
             return;
         }
     }
@@ -254,7 +231,7 @@ void run(GIMP_CONST gchar *name,
 
     ufraw_config(uf, &rc, NULL, NULL);
     sendToGimpMode = (uf->conf->createID == send_id);
-#if !defined(UFRAW_CINEPAINT) && !HAVE_GIMP_2_9
+#if !HAVE_GIMP_2_9
     if (loadThumbnail) {
         uf->conf->size = size;
         uf->conf->embeddedImage = TRUE;
@@ -271,7 +248,7 @@ void run(GIMP_CONST gchar *name,
             !loadThumbnail && !sendToGimpMode) {
         /* Show the preview in interactive mode, unless if we are
          * in thumbnail mode or 'send to gimp' mode. */
-        status = ufraw_preview(uf, &rc, PLUGIN_MODE, ufraw_save_gimp_image);
+        status = ufraw_preview(uf, &rc, ufraw_gimp_plugin, ufraw_save_gimp_image);
     } else {
         if (sendToGimpMode) {
             char *text = g_strdup_printf(_("Loading raw file '%s'"),
@@ -284,7 +261,9 @@ void run(GIMP_CONST gchar *name,
         if (status != UFRAW_SUCCESS) {
             values[0].type = GIMP_PDB_STATUS;
             values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+#ifndef _WIN32
             gdk_threads_leave();
+#endif
             return;
         }
         if (sendToGimpMode) gimp_progress_update(0.3);
@@ -304,7 +283,9 @@ void run(GIMP_CONST gchar *name,
             values[0].data.d_status = GIMP_PDB_CANCEL;
         else
             values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+#ifndef _WIN32
         gdk_threads_leave();
+#endif
         return;
     }
     *nreturn_vals = 2;
@@ -319,7 +300,9 @@ void run(GIMP_CONST gchar *name,
         values[3].type = GIMP_PDB_INT32;
         values[3].data.d_int32 = uf->initialHeight;
     }
+#ifndef _WIN32
     gdk_threads_leave();
+#endif
     return;
 }
 
@@ -369,7 +352,7 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
         if (ufraw_convert_image(uf) != UFRAW_SUCCESS)
             return UFRAW_ERROR;
         ufraw_get_scaled_crop(uf, &Crop);
-#if defined(UFRAW_CINEPAINT) || HAVE_GIMP_2_9
+#if HAVE_GIMP_2_9
         if (uf->conf->profile[out_profile]
                 [uf->conf->profileIndex[out_profile]].BitDepth == 16)
             depth = 6;
@@ -381,12 +364,11 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
     }
 #if HAVE_GIMP_2_9
     uf->gimpImage =
-        gimp_image_new_with_precision(Crop.width, Crop.height,
-                                      DEPTH_TO_BASETYPE(depth),
-                                      DEPTH_TO_PRECISION(depth));
+        gimp_image_new_with_precision(Crop.width, Crop.height, GIMP_RGB,
+                                      depth == 3 ? GIMP_PRECISION_U8_GAMMA :
+                                      GIMP_PRECISION_U16_GAMMA);
 #else
-    uf->gimpImage = gimp_image_new(Crop.width, Crop.height,
-                                   DEPTH_TO_BASETYPE(depth));
+    uf->gimpImage = gimp_image_new(Crop.width, Crop.height, GIMP_RGB);
 #endif
     if (uf->gimpImage == -1) {
         ufraw_message(UFRAW_ERROR, _("Can't allocate new image."));
@@ -396,16 +378,12 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 
     /* Create the "background" layer to hold the image... */
     layer = gimp_layer_new(uf->gimpImage, _("Background"), Crop.width,
-                           Crop.height, DEPTH_TO_IMAGETYPE(depth),
-                           100.0, GIMP_NORMAL_MODE);
-#ifdef UFRAW_CINEPAINT
-    gimp_image_add_layer(uf->gimpImage, layer, 0);
-#else
+                           Crop.height, GIMP_RGB_IMAGE, 100.0,
+                           GIMP_NORMAL_MODE);
 #if defined(GIMP_CHECK_VERSION) && GIMP_CHECK_VERSION(2,7,3)
     gimp_image_insert_layer(uf->gimpImage, layer, 0, 0);
 #else
     gimp_image_add_layer(uf->gimpImage, layer, 0);
-#endif
 #endif
 
     /* Get the drawable and set the pixel region for our load... */
@@ -449,27 +427,6 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 
     if (uf->conf->embeddedImage) return UFRAW_SUCCESS;
 
-#ifdef UFRAW_CINEPAINT
-    /* No parasites in CinePaint.
-     * We need to disable EXIF export.
-     * ICC profile export works differently. */
-    if (strcmp(uf->developer->profileFile[out_profile], "") == 0) {
-        gimp_image_set_srgb_profile(uf->gimpImage);
-    } else {
-        char *buf;
-        gsize len;
-        if (g_file_get_contents(uf->developer->profileFile[out_profile],
-                                &buf, &len, NULL)) {
-            gimp_image_set_icc_profile_by_mem(uf->gimpImage, len, buf,
-                                              ICC_IMAGE_PROFILE);
-            g_free(buf);
-        } else {
-            ufraw_message(UFRAW_WARNING,
-                          _("Failed to embed output profile '%s' in image."),
-                          uf->developer->profileFile[out_profile]);
-        }
-    }
-#else
     ufraw_exif_prepare_output(uf);
     if (uf->outputExifBuf != NULL) {
         if (uf->outputExifBufLen > 65533) {
@@ -478,10 +435,31 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
                           uf->outputExifBufLen);
         } else {
             GimpParasite *exif_parasite;
+
             exif_parasite = gimp_parasite_new("exif-data",
                                               GIMP_PARASITE_PERSISTENT, uf->outputExifBufLen, uf->outputExifBuf);
+#if defined(GIMP_CHECK_VERSION) && GIMP_CHECK_VERSION(2,8,0)
+            gimp_image_attach_parasite(uf->gimpImage, exif_parasite);
+#else
             gimp_image_parasite_attach(uf->gimpImage, exif_parasite);
+#endif
             gimp_parasite_free(exif_parasite);
+
+#if defined(GIMP_CHECK_VERSION) && GIMP_CHECK_VERSION(2,8,0)
+            {
+                GimpParam    *return_vals;
+                gint          nreturn_vals;
+                return_vals = gimp_run_procedure("plug-in-metadata-decode-exif",
+                                                 &nreturn_vals,
+                                                 GIMP_PDB_IMAGE, uf->gimpImage,
+                                                 GIMP_PDB_INT32, 7,
+                                                 GIMP_PDB_INT8ARRAY, "unused",
+                                                 GIMP_PDB_END);
+                if (return_vals[0].data.d_status != GIMP_PDB_SUCCESS) {
+                    g_warning("UFRaw Exif -> XMP Merge failed");
+                }
+            }
+#endif
         }
     }
     /* Create "icc-profile" parasite from output profile
@@ -494,7 +472,11 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
             GimpParasite *icc_parasite;
             icc_parasite = gimp_parasite_new("icc-profile",
                                              GIMP_PARASITE_PERSISTENT, len, buf);
+#if defined(GIMP_CHECK_VERSION) && GIMP_CHECK_VERSION(2,8,0)
+            gimp_image_attach_parasite(uf->gimpImage, icc_parasite);
+#else
             gimp_image_parasite_attach(uf->gimpImage, icc_parasite);
+#endif
             gimp_parasite_free(icc_parasite);
             g_free(buf);
         } else {
@@ -503,7 +485,6 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
                           uf->developer->profileFile[out_profile]);
         }
     }
-#endif
     return UFRAW_SUCCESS;
 }
 
