@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_lensfun.cc - define lensfun UFObject settings.
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -675,7 +675,7 @@ void Lensfun::SetLensInterpolation()
     double crop_factor = 1.0;
     int count = sscanf(model, "Crop factor %lf", &crop_factor);
     if ((strcmp(make, "Generic") == 0 && count == 1) ||
-            (strcmp(make, "") == 0 && strcmp(model, "") == 0)) {
+            (strcasecmp(make, "") == 0 && strcmp(model, "") == 0)) {
         lfLens cropLens;
         cropLens.SetMaker(make);
         cropLens.SetModel(model);
@@ -873,7 +873,8 @@ extern "C" {
         return new Lensfun();
     }
 
-    const struct lfCamera *ufraw_lensfun_camera(const UFObject *lensfun) {
+    const struct lfCamera *ufraw_lensfun_camera(const UFObject *lensfun)
+    {
         return &static_cast<const UFRaw::Lensfun *>(lensfun)->Camera;
     }
 
@@ -882,7 +883,8 @@ extern "C" {
         static_cast<UFRaw::Lensfun *>(lensfun)->SetCamera(*camera);
     }
 
-    const struct lfLens *ufraw_lensfun_interpolation_lens(const UFObject *lensfun) {
+    const struct lfLens *ufraw_lensfun_interpolation_lens(const UFObject *lensfun)
+    {
         return &static_cast<const UFRaw::Lensfun *>(lensfun)->Interpolation;
     }
 
