@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_settings.cc - define all UFObject settings.
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -354,7 +354,7 @@ void Image::SetUFRawData(ufraw_data *data)
         return;
 
     dcraw_data *raw = static_cast<dcraw_data *>(uf->raw);
-    if (strcmp(uf->conf->make, raw->make) != 0 ||
+    if (strcasecmp(uf->conf->make, raw->make) != 0 ||
             strcmp(uf->conf->model, raw->model) != 0)
         uf->WBDirty = TRUE; // Re-calculate channel multipliers.
     if (uf->LoadingID)
@@ -367,7 +367,7 @@ void Image::SetUFRawData(ufraw_data *data)
     const wb_data *lastPreset = NULL;
     uf->wb_presets_make_model_match = FALSE;
     char model[max_name];
-    if (strcmp(uf->conf->make, "MINOLTA") == 0 &&
+    if (strcasecmp(uf->conf->make, "Minolta") == 0 &&
             (strncmp(uf->conf->model, "ALPHA", 5) == 0 ||
              strncmp(uf->conf->model, "MAXXUM", 6) == 0)) {
         /* Canonize Minolta model names (copied from dcraw) */
@@ -395,7 +395,7 @@ void Image::SetUFRawData(ufraw_data *data)
                 }
             }
             wb << new UFString(ufPreset, wb_preset[i].name);
-        } else if (strcmp(wb_preset[i].make, uf->conf->make) == 0 &&
+        } else if (strcasecmp(wb_preset[i].make, uf->conf->make) == 0 &&
                    strcmp(wb_preset[i].model, model) == 0) {
             /* Camera specific presets */
             uf->wb_presets_make_model_match = TRUE;
@@ -492,7 +492,8 @@ extern "C" {
         dynamic_cast<UFRaw::Image *>(obj)->SetUFRawData(uf);
     }
 
-    struct ufraw_struct *ufraw_image_get_data(UFObject *obj) {
+    struct ufraw_struct *ufraw_image_get_data(UFObject *obj)
+    {
         return UFRaw::Image::UFRawData(obj);
     }
 
