@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_ufraw.c - program interface to all the components
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
 void ufraw_prepare_tca(ufraw_data *uf);
 #endif
 static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
-        const char *formats, const char *caller);
+                               const char *formats, const char *caller);
 static void ufraw_convert_image_raw(ufraw_data *uf, UFRawPhase phase);
 static void ufraw_convert_image_first(ufraw_data *uf, UFRawPhase phase);
 static void ufraw_convert_image_transform(ufraw_data *uf, ufraw_image_data *img,
@@ -261,6 +261,7 @@ ufraw_data *ufraw_open(char *filename)
     uf->displayProfileSize = 0;
     uf->RawHistogram = NULL;
     uf->HaveFilters = raw->filters != 0;
+    uf->IsXTrans = raw->filters == 9;
 #ifdef HAVE_LENSFUN
     uf->modFlags = 0;
     uf->TCAmodifier = NULL;
@@ -667,7 +668,7 @@ int ufraw_load_raw(ufraw_data *uf)
     }
     uf->raw_multiplier = ufraw_scale_raw(raw);
     /* Canon EOS cameras require special exposure normalization */
-    if (strcmp(uf->conf->make, "Canon") == 0 &&
+    if (strcasecmp(uf->conf->make, "Canon") == 0 &&
             strncmp(uf->conf->model, "EOS", 3) == 0) {
         int c, max = raw->cam_mul[0];
         for (c = 1; c < raw->colors; c++) max = MAX(raw->cam_mul[c], max);
@@ -999,7 +1000,7 @@ reduction(+:count) \
 }
 
 static void ufraw_despeckle_line(guint16 *base, int step, int size, int window,
-        double decay, int colors, int c)
+                                 double decay, int colors, int c)
 {
     unsigned lum[size];
     int i, j, start, end, next, v, cold, hot, coldj, hotj, fix;
@@ -1129,9 +1130,9 @@ static int ufraw_calculate_scale(ufraw_data *uf)
         scale = uf->conf->shrink * MIN(raw->pixel_aspect, 1 / raw->pixel_aspect);
     } else if (uf->conf->interpolation == half_interpolation) {
         scale = 2;
-    /* Wanted size is smaller than raw size (size is after a raw->shrink)
-     * (assuming there are filters). */
-    } else if (uf->conf->size > 0 && uf->HaveFilters) {
+        /* Wanted size is smaller than raw size (size is after a raw->shrink)
+         * (assuming there are filters). */
+    } else if (uf->conf->size > 0 && uf->HaveFilters && !uf->IsXTrans) {
         int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
         int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
         int cropSize = MAX(cropHeight, cropWidth);
@@ -1202,7 +1203,7 @@ static void ufraw_convert_image_raw(ufraw_data *uf, UFRawPhase phase)
     rawimage = raw->raw.image;
     raw->raw.image = (dcraw_image_type *)img->buffer;
     /* The threshold is scaled for compatibility */
-    dcraw_wavelet_denoise(raw, uf->conf->threshold * sqrt(uf->raw_multiplier));
+    if (!uf->IsXTrans) dcraw_wavelet_denoise(raw, uf->conf->threshold * sqrt(uf->raw_multiplier));
     dcraw_finalize_raw(raw, dark, uf->developer->rgbWB);
     raw->raw.image = rawimage;
     ufraw_despeckle(uf, phase);
@@ -1236,6 +1237,8 @@ static void ufraw_convert_image_first(ufraw_data *uf, UFRawPhase phase)
     ufraw_convertshrink(uf, &final);
     raw->raw.image = rawimage;
     dcraw_flip_image(&final, uf->conf->orientation);
+    /* The threshold is scaled for compatibility */
+    if (uf->IsXTrans) dcraw_wavelet_denoise_shrinked(&final, uf->conf->threshold * sqrt(uf->raw_multiplier));
 
     // The 'out' image contains the predicted image dimensions.
     // We want to be sure that our predictions were correct.
@@ -1317,8 +1320,8 @@ static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
                 float srcX = modcoord[0];
                 float srcY = modcoord[1];
                 /* Do it in integer arithmetic, it's faster */
-                int xx = (int)srcX;
-                int yy = (int)srcY;
+                int xx = (int)floor(srcX + 0.5);
+                int yy = (int)floor(srcY + 0.5);
                 // TODO: better handling of the borders.
                 if (xx < 0 || yy < 0 || xx + 1 >= img->width || yy + 1 >= img->height) {
                     dst[c] = 0;
@@ -1356,7 +1359,7 @@ static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase,
 }
 
 static void ufraw_image_init(ufraw_image_data *img,
-        int width, int height, int bitdepth)
+                             int width, int height, int bitdepth)
 {
     if (img->height == height && img->width == width &&
             img->depth == bitdepth && img->buffer != NULL)
@@ -1574,7 +1577,7 @@ static void ufraw_convert_prepare_buffers(ufraw_data *uf, UFRawPhase phase)
  * constants by variables may turn off some compiler optimizations.
  */
 static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
-        const char *formats, const char *caller)
+                               const char *formats, const char *caller)
 {
     int b, c;
 
@@ -1607,7 +1610,7 @@ static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
 }
 
 ufraw_image_data *ufraw_get_image(ufraw_data *uf, UFRawPhase phase,
-        gboolean bufferok)
+                                  gboolean bufferok)
 {
     ufraw_convert_prepare_buffers(uf, phase);
     // Find the closest phase that is actually rendered:
@@ -1956,6 +1959,7 @@ int ufraw_set_wb(ufraw_data *uf)
     /* For uf_manual_wb we calculate chanMul from the temperature/green. */
     /* For all other it is the other way around. */
     if (ufarray_is_equal(wb, uf_manual_wb)) {
+        double chanMulArray[4] = {1, 1, 1, 1 };
         Temperature_to_RGB(ufnumber_value(temperature), rgbWB);
         rgbWB[1] = rgbWB[1] / ufnumber_value(green);
         /* Suppose we shot a white card at some temperature:
@@ -1973,12 +1977,11 @@ int ufraw_set_wb(ufraw_data *uf)
          */
         if (uf->raw_color) {
             /* If there is no color matrix it is simple */
-            double chanMulArray[4];
-            for (c = 0; c < 3; c++)
-                chanMulArray[c] = raw->pre_mul[c] / rgbWB[c];
+            if (uf->colors > 1)
+                for (c = 0; c < 3; c++)
+                    chanMulArray[c] = raw->pre_mul[c] / rgbWB[c];
             ufnumber_array_set(chanMul, chanMulArray);
         } else {
-            double chanMulArray[4] = {1, 1, 1, 1 };
             for (c = 0; c < uf->colors; c++) {
                 double chanMulInv = 0;
                 for (cc = 0; cc < 3; cc++)
@@ -2036,7 +2039,7 @@ int ufraw_set_wb(ufraw_data *uf)
     } else {
         int lastTuning = -1;
         char model[max_name];
-        if (strcmp(uf->conf->make, "MINOLTA") == 0 &&
+        if (strcasecmp(uf->conf->make, "Minolta") == 0 &&
                 (strncmp(uf->conf->model, "ALPHA", 5) == 0 ||
                  strncmp(uf->conf->model, "MAXXUM", 6) == 0)) {
             /* Canonize Minolta model names (copied from dcraw) */
@@ -2047,7 +2050,7 @@ int ufraw_set_wb(ufraw_data *uf)
         }
         for (i = 0; i < wb_preset_count; i++) {
             if (ufarray_is_equal(wb, wb_preset[i].name) &&
-                    !strcmp(uf->conf->make, wb_preset[i].make) &&
+                    !strcasecmp(uf->conf->make, wb_preset[i].make) &&
                     !strcmp(model, wb_preset[i].model)) {
                 if (ufnumber_value(wbTuning) == wb_preset[i].tuning) {
                     double chanMulArray[4] = {1, 1, 1, 1 };
