@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_writer.c - functions to output image files in different formats.
- * Copyright 2004-2013 by Udi Fuchs
+ * Copyright 2004-2014 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,23 @@
 #include <glib/gi18n.h>
 #include <errno.h>	/* for errno */
 #include <string.h>
+#ifdef HAVE_LCMS2
+#include <lcms2.h>
+#else
 #include <lcms.h>
+typedef DWORD cmsUInt32Number;
+static LCMSBOOL cmsSaveProfileToMem(cmsHPROFILE hProfile, void *MemPtr,
+                                    cmsUInt32Number *BytesNeeded)
+{
+    size_t _BytesNeeded;
+    LCMSBOOL retval;
+
+    retval = _cmsSaveProfileToMem(hProfile, MemPtr, &_BytesNeeded);
+    if (BytesNeeded)
+        *(BytesNeeded) = (cmsUInt32Number) _BytesNeeded;
+    return retval;
+}
+#endif
 #ifdef HAVE_LIBTIFF
 #include <tiffio.h>
 #endif
@@ -204,7 +220,8 @@ int png_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
 
 #if defined(HAVE_LIBCFITSIO) && defined(_WIN32)
 /* localtime_r() is not included in the _WIN32 API. */
-static struct tm *localtime_r(const time_t *timep, struct tm *result) {
+static struct tm *localtime_r(const time_t *timep, struct tm *result)
+{
     struct tm *p = localtime(timep);
     memset(result, 0, sizeof(*result));
     if (p) {
@@ -259,7 +276,8 @@ int ufraw_write_image(ufraw_data *uf)
     fitsfile *fitsFile;
 #endif
     char * volatile confFilename = NULL;
-    int grayscaleMode = uf->conf->grayscaleMode != grayscale_none;
+    int volatile grayscaleMode = uf->conf->grayscaleMode != grayscale_none ||
+                                 uf->colors == 1;
     ufraw_message_reset(uf);
 
     if (uf->conf->createID == only_id ||
@@ -383,7 +401,7 @@ int ufraw_write_image(ufraw_data *uf)
                      ? PHOTOMETRIC_MINISBLACK : PHOTOMETRIC_RGB);
 #ifdef HAVE_LIBZ
         if (uf->conf->losslessCompress) {
-            TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_DEFLATE);
+            TIFFSetField(out, TIFFTAG_COMPRESSION, COMPRESSION_ADOBE_DEFLATE);
             TIFFSetField(out, TIFFTAG_ZIPQUALITY, 9);
             TIFFSetField(out, TIFFTAG_PREDICTOR, 2);
         } else
@@ -405,11 +423,11 @@ int ufraw_write_image(ufraw_data *uf)
             }
         } else if (uf->conf->profileIndex[out_profile] == 1) { // Embed sRGB.
             cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
-            gsize len = 0;
-            _cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+            cmsUInt32Number len = 0;
+            cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
             if (len > 0) {
                 unsigned char buf[len];
-                _cmsSaveProfileToMem(hOutProfile, buf, &len);
+                cmsSaveProfileToMem(hOutProfile, buf, &len);
                 TIFFSetField(out, TIFFTAG_ICCPROFILE, len, buf);
             } else {
                 ufraw_set_warning(uf,
@@ -478,11 +496,11 @@ int ufraw_write_image(ufraw_data *uf)
             }
         } else if (uf->conf->profileIndex[out_profile] == 1) { // Embed sRGB.
             cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
-            gsize len = 0;
-            _cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+            cmsUInt32Number len = 0;
+            cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
             if (len > 0) {
                 unsigned char buf[len];
-                _cmsSaveProfileToMem(hOutProfile, buf, &len);
+                cmsSaveProfileToMem(hOutProfile, buf, &len);
                 write_icc_profile(&cinfo, buf, len);
             } else {
                 ufraw_set_warning(uf,
@@ -570,11 +588,11 @@ int ufraw_write_image(ufraw_data *uf)
                 }
             } else if (uf->conf->profileIndex[out_profile] == 1) { // Embed sRGB.
                 cmsHPROFILE hOutProfile = cmsCreate_sRGBProfile();
-                gsize len = 0;
-                _cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
+                cmsUInt32Number len = 0;
+                cmsSaveProfileToMem(hOutProfile, 0, &len); // Calculate len.
                 if (len > 0) {
                     char buf[len];
-                    _cmsSaveProfileToMem(hOutProfile, buf, &len);
+                    cmsSaveProfileToMem(hOutProfile, buf, &len);
                     png_set_iCCP(png, info,
                                  uf->conf->profile[out_profile]
                                  [uf->conf->profileIndex[out_profile]].name,
